@@ -20,11 +20,11 @@ use vars qw($VERSION @EXPORT @EXPORT_OK %EXPORT_TAGS @ISA);
 
 #use Data::Dumper;
 
-$VERSION = '0.08';
+$VERSION = '0.09';
 @ISA = qw(Exporter);
 @EXPORT = qw();
 @EXPORT_OK = qw(
-sethtml clearhtml parse_text print printheader printfooter
+sethtml clearhtml parse_text parse_doi print printheader printfooter
 doi score date atitle jtitle volume issue genre spage epage authcount auth query
 );
 %EXPORT_TAGS = (all => \@EXPORT_OK);
@@ -255,6 +255,116 @@ sub parse_text {
   }
 }
 
+sub parse_doi {
+  # given a DOI, use unixref interface to convert into a full citation
+  my ($self, $doi) = @_;
+  
+  my $req = HTTP::Request->new(GET =>'http://dx.doi.org/'.$doi,['Accept' =>'application/vnd.crossref.unixsd+xml']);
+  my $ua = LWP::UserAgent->new;
+  my $res = $ua->request($req);
+  if ($res->is_success) {
+    # now parse the xml
+    my $xs = XML::Simple->new();
+    my $data = $xs->XMLin($res->decoded_content);
+    my $cite =  $data->{'query_result'}->{'body'}->{'query'}->{'doi_record'}->{'crossref'};
+    my $cc = undef;
+    if (exists($cite->{'conference'})) {
+      $self->_setgenre('proceeding');
+      if (exists($cite->{'conference'}->{'proceedings_metadata'})) {
+        $self->_setjtitle($cite->{'conference'}->{'proceedings_metadata'}->{'proceedings_title'});
+        if (exists $cite->{'conference'}->{'proceedings_metadata'}->{'publication_date'}) {$self->_setdate($cite->{'conference'}->{'proceedings_metadata'}->{'publication_date'}->{'year'});}
+      } else {
+        $self->_setjtitle($cite->{'conference'}->{'proceedings_series_metadata'}->{'series_metadata'}->{'proceedings_title'});
+        $self->_setvolume($cite->{'conference'}->{'proceedings_series_metadata'}->{'series_metadata'}->{'volume'});
+        if (exists $cite->{'conference'}->{'proceedings_series_metadata'}->{'publication_date'}) {$self->_setdate($cite->{'conference'}->{'proceedings_series_metadata'}->{'publication_date'}->{'year'});}
+      }
+      $cc = $cite->{'conference'}->{'conference_paper'};
+    } elsif (exists($cite->{'journal'})) {
+      $self->_setgenre('article');
+      $self->_setjtitle($cite->{'journal'}->{'journal_metadata'}->{'full_title'});
+      $cc = $cite->{'journal'}->{'journal_issue'};
+      if (exists($cc->{'journal_volume'})) {$self->_setvolume($cc->{'journal_volume'}->{'volume'});}
+      if (exists($cc->{'issue'})) {$self->_setissue($cc->{'issue'});}
+      $cc = $cite->{'journal'}->{'journal_article'};
+    } elsif (exists($cite->{'book'})) {
+      if ($cite->{'book'}->{'book_type'} ne 'other' ) {
+        $self->_setgenre('book');
+      } else {
+        $self->_setgenre('bookitem');
+      }
+      my $jtitle = '';
+      if (exists($cite->{'book'}->{'book_series_metadata'})) {
+        if (exists($cite->{'book'}->{'book_series_metadata'}->{'titles'}->{'title'})) {
+          $jtitle .= $cite->{'book'}->{'book_series_metadata'}->{'titles'}->{'title'}.': ';
+        }
+        $jtitle .= $cite->{'book'}->{'book_series_metadata'}->{'series_metadata'}->{'titles'}->{'title'};
+        $self->_setvolume($cite->{'book'}->{'book_series_metadata'}->{'volume'});
+        if (exists $cite->{'book'}->{'book_series_metadata'}->{'publication_date'}) {$self->_setdate($cite->{'book'}->{'book_series_metadata'}->{'publication_date'}->{'year'});}
+      } elsif (exists($cite->{'book'}->{'book_metadata'})) {
+        $jtitle .= $cite->{'book'}->{'book_metadata'}->{'titles'}->{'title'};
+        if (exists($cite->{'book'}->{'book_metadata'}->{'series_metadata'}->{'titles'}->{'title'})) {
+          $jtitle .= ": ".$cite->{'book'}->{'book_metadata'}->{'series_metadata'}->{'titles'}->{'title'};
+          if (exists $cite->{'book'}->{'book_metadata'}->{'series_metadata'}->{'volume'}) {$self->_setvolume($cite->{'book'}->{'book_metadata'}->{'series_metadata'}->{'volume'});}
+        }
+        if (exists $cite->{'book'}->{'book_metadata'}->{'volume'}) {$self->_setvolume($cite->{'book'}->{'book_metadata'}->{'volume'});}
+        if (exists $cite->{'book'}->{'book_metadata'}->{'publication_date'}) {$self->_setdate($cite->{'book'}->{'book_metadata'}->{'publication_date'}->{'year'});}
+      } else {
+        if (exists($cite->{'book'}->{'book_set_metadata'}->{'titles'}->{'title'})) {
+          $jtitle .= $cite->{'book'}->{'book_set_metadata'}->{'titles'}->{'title'}.': ';
+        }
+        $jtitle .= $cite->{'book'}->{'book_set_metadata'}->{'set_metadata'}->{'titles'}->{'title'};
+      }
+      $self->_setjtitle($jtitle);
+      $cc = $cite->{'book'}->{'content_item'};
+    } else {
+        # something else -- might be dissertation, report-paper, standard, sa-component, database
+        # fall back to alternative interface for now
+        $self->parse_text($doi);
+        return; # stop here
+    }
+    $self->_setscore(1);
+    $self->_setquery($doi);
+    $self->_setdoi($doi);
+    if (!defined $cc) {
+      # seems like an incomplete entry
+      return;
+    }
+    $self->_setatitle($cc->{'titles'}->{'title'});
+    $self->_setdoi($cc->{'doi_data'}->{'doi'});
+    if (ref($cc->{'publication_date'}) eq "HASH") {
+      $self->_setdate($cc->{'publication_date'}->{'year'});
+    } else { # we have multiple dates, lets try and pick put the print date
+      my $found = 0; my $count=0;
+      foreach my $d (@{$cc->{'publication_date'}}) {
+        if ($d->{'media_type'} eq 'print') {
+          $self->_setdate($d->{'year'}); $found = 1;
+        }
+        $count++;
+      }
+      if ((!$found) && ($count>0)) {$self->_setdate(${$cc->{'publication_date'}}[0]->{'year'});}
+    }
+    if (exists(${$cc}{'pages'})) {
+      if (exists(${$cc->{'pages'}}{'first_page'})) {$self->_setspage($cc->{'pages'}->{'first_page'});}
+      if (exists(${$cc->{'pages'}}{'last_page'})) {$self->_setepage($cc->{'pages'}->{'last_page'});}
+    }
+    $cc = $cc->{'contributors'}->{'person_name'};
+    if (ref($cc) eq "HASH") {
+      $self->_setauthcount(1);
+      $self->_setauth(1,$cc->{'given_name'}.' '.$cc->{'surname'});
+    } else {
+      my $count = 0;
+      foreach my $au (@{$cc}) {
+        if ($au->{'contributor_role'} ne 'author') {next;}
+        $count++;
+        $self->_setauth($count, $au->{'given_name'}.' '.$au->{'surname'});
+      }
+      $self->_setauthcount($count);
+    }
+  } else {
+    $self->_err("Problem with search.crossref.org/guestquery: ".$res->status_line);
+  }
+}
+
 sub printheader {
   my $self = shift @_;
   my $id = shift @_;
@@ -430,7 +540,15 @@ Creates a new Bib::CrossRef object
 
  $ref->parse_text($string)
 
-Provides a text string that Bib::CrossRef will try to resolve into a full citation with the help of crossref.org
+Given a text string, Bib::CrossRef will try to resolve into a full citation with the help of crossref.org
+
+=head2 parse_doi
+
+ $ref->parse_doi($doi)
+
+Given a string containing a DOI e.g. 10.1109/tnet.2012.2202686, resolve into a full citation with the unixref
+interface of crossref.org.  This should be definitive publishers DOI and is usually
+much as the same as calling $ref->parse_text($doi), but is included completeness.
 
 =head2 doi
 
@@ -550,7 +668,7 @@ When html formatting is enabled, prints some html footer tags
  
 You can export the following functions if you do not want to use the object orientated interface:
 
-sethtml clearhtml set_details print printheader printfooter
+parse_text parse_doi sethtml clearhtml set_details print printheader printfooter
 doi score date atitle jtitle volume issue genre spage epage authcount auth query url
 
 The tag C<all> is available to easily export everything:
@@ -559,7 +677,7 @@ use Bib::CrossRef qw(:all);
 
 =head1 VERSION
  
-Ver 0.08
+Ver 0.09
  
 =head1 AUTHOR
  
